from __future__ import print_function
from tensorflow.keras.models import Model
from tensorflow.keras.layers import Input, LSTM, Dense,  Add
import numpy as np
from weather import Weather
from sklearn.preprocessing import MinMaxScaler
from tensorflow.keras.models import load_model
import pandas as pd
import matplotlib.pyplot as plt



m = Weather()
# m.station = 'elberta'
m.station = 'atmore'
# m.test_feature_list = ["RH_2m"]
m.test_feature_list = ["PTemp"]
# m.test_feature_list = ["WndDir_2m"]
# m.test_feature_list = ["WndSpd_2m"]
# m.test_feature_list = ["Pressure_1"]
# m.test_feature_list = ["Precip_TB3_Tot"]

import argparse
import sys

# args=sys.argv[1:]

# m.station = args[0]
# m.test_feature_list = [args[1]]




global_path = "cache/"+m.station+"_LSTM_"+m.test_feature_list[0]+"_all"


train = True
duration = 9
year_duration = 1
start_date = "0301"


train_x, train_y = m.read_lstm_file(year = "2018", year_duration = year_duration, start_date = start_date, duration = duration, shift=5, seq = 5)


train_wrf_x = m.read_wrf_file(year = "2018", year_duration = year_duration, start_date = start_date, duration = duration, shift=5, seq = 5)

x_scaler = MinMaxScaler()
x_scaler.fit(train_x)
x_train = x_scaler.transform(train_x)



y_scaler = MinMaxScaler()
y_scaler.fit(train_y)
y_train = y_scaler.transform(train_y)


wrf_scaler =  MinMaxScaler()
wrf_scaler.fit(train_wrf_x)
x_train_wrf = wrf_scaler.transform(train_wrf_x)



y_train_L = y_train.reshape(1, y_train.shape[0])[0]
x_train = np.array(x_train)
y_train_L = np.array(y_train_L)



encoder_input_data, decoder_input_data, decoder_target_data = m.pre_seq(x_train, y_train_L)
encoder_wrf_input_data, _, _ =  m.pre_seq(x_train_wrf, y_train_L)





duration = 1
start_date = "0308"
test_x, test_y = m.read_lstm_file(year = "2019", year_duration = 1, start_date = start_date, duration = duration, shift=5, seq = 5)
test_wrf_x = m.read_wrf_file(year = "2019", year_duration = 1, start_date = start_date, duration = duration, shift=5, seq = 5)




x_test = x_scaler.transform(test_x)
y_test = y_scaler.transform(test_y)
x_test_wrf = wrf_scaler.transform(test_wrf_x)






y_test_L = y_test.reshape(1, y_test.shape[0])[0]
x_test = np.array(x_test)
y_test_L = np.array(y_test_L)



test_encoder_input_data, test_decoder_input_data, test_decoder_target_data = m.pre_seq(x_test, y_test_L)
test_encoder_wrf_input_data, _, _ =  m.pre_seq(x_test_wrf, y_test_L)





batch_size = 64  # Batch size for training.
epochs = 5  # Number of epochs to train for.
latent_dim = 256  # Latent dimensionality of the encoding space.

# Vectorize the data.

input_texts = encoder_input_data
input_len = len(input_texts)
target_texts = []


num_encoder_tokens = len(m.feature_list_full_value)
num_wrf_encoder_tokens = len(m.wrf_feature_list_value)
num_decoder_tokens = len(m.test_feature_list)+2
max_encoder_seq_length = 12
max_decoder_seq_length = 6+1


# Define an input sequence and process it.
encoder_inputs = Input(shape=(None, num_encoder_tokens))
encoder = LSTM(latent_dim, return_state=True)
encoder_outputs, state_h, state_c = encoder(encoder_inputs)
# We discard `encoder_outputs` and only keep the states.
encoder_states = [state_h, state_c]


# Define wrf encoder
wrf_encoder_inputs = Input(shape=(None, num_wrf_encoder_tokens))
wrf_encoder = LSTM(latent_dim, return_state=True)
wrf_encoder_outputs, wrf_state_h, wrf_state_c = wrf_encoder(wrf_encoder_inputs)
wrf_encoder_states = [wrf_state_h, wrf_state_c]


h_add =  Add()([state_h, wrf_state_h])
c_add =  Add()([state_c, wrf_state_c])


all_encoder_states = [h_add, c_add]


# Set up the decoder, using `encoder_states` as initial state.
decoder_inputs = Input(shape=(None, num_decoder_tokens))
# We set up our decoder to return full output sequences,
# and to return internal states as well. We don't use the
# return states in the training model, but we will use them in inference.
decoder_lstm = LSTM(latent_dim, return_sequences=True, return_state=True)
decoder_outputs, _, _ = decoder_lstm(decoder_inputs, initial_state=all_encoder_states)
decoder_dense = Dense(num_decoder_tokens, activation='sigmoid')
decoder_outputs = decoder_dense(decoder_outputs)


# Define the model that will turn
# `encoder_input_data` & `decoder_input_data` into `decoder_target_data`
model = Model([encoder_inputs, wrf_encoder_inputs, decoder_inputs], decoder_outputs)
# plot_model(model, to_file='model.png', show_shapes=True)



# Run training
model.compile(optimizer='adam', loss='mean_squared_error')

if train:
    for ii in range(2):
        print(str(ii)+'/'+str(5))
        model.fit([encoder_input_data, encoder_wrf_input_data, decoder_input_data], decoder_target_data,batch_size=batch_size, epochs=epochs, validation_split=0.2, verbose=0)
        # Save model
        model.save(global_path+'.model')

model = load_model(global_path+'.model')


# Next: inference mode (sampling).
# Here's the drill:
# 1) encode input and retrieve initial decoder state
# 2) run one step of decoder with this initial state
# and a "start of sequence" token as target.
# Output will be the next target token
# 3) Repeat with the current target token and current states
# Define sampling models
encoder_model = Model(encoder_inputs, encoder_states)
wrf_encoder_model = Model(wrf_encoder_inputs, wrf_encoder_states)




decoder_state_input_h = Input(shape=(latent_dim,))
decoder_state_input_c = Input(shape=(latent_dim,))
decoder_states_inputs = [decoder_state_input_h, decoder_state_input_c]
decoder_outputs, state_h, state_c = decoder_lstm(decoder_inputs, initial_state=decoder_states_inputs)
decoder_states = [state_h, state_c]
decoder_outputs = decoder_dense(decoder_outputs)
decoder_model = Model([decoder_inputs] + decoder_states_inputs, [decoder_outputs] + decoder_states)


if train:
    encoder_model.save(global_path+'.encoder')
    decoder_model.save(global_path+'.decoder')
    wrf_encoder_model.save(global_path+'_wrfencoder')


encoder_model = load_model(global_path+'.encoder')
decoder_model = load_model(global_path+'.decoder')
wrf_encoder_model = load_model(global_path+'_wrfencoder')




def decode_sequence(input_seq, wrf_input_seq):
    # Encode the input as state vectors.
    states_value_h, states_value_c = encoder_model.predict(input_seq)
    wrf_states_value_h, wrf_states_value_c = wrf_encoder_model.predict(wrf_input_seq)


    all_h = states_value_h+wrf_states_value_h
    all_c = states_value_c+wrf_states_value_c

    states_value = [all_h, all_c]

    target_seq = np.zeros((1, 1, num_decoder_tokens))

    stop_condition = False
    decoded_sentence = []
    while not stop_condition:
            output_tokens, h, c = decoder_model.predict([target_seq] + states_value)
  
            pre_tmp_value = output_tokens[0,0,0]

            decoded_sentence.append(pre_tmp_value)
            
            end_bit = output_tokens[0,0,-1]
    
            # Exit condition: either hit max length
            # or find stop character.

            if (end_bit > 1 or len(decoded_sentence) > max_decoder_seq_length-2):
                stop_condition = True

            # Update the target sequence (of length 1).
            target_seq = np.zeros((1, 1, num_decoder_tokens))
            target_seq[0, 0, 0] = pre_tmp_value
            # Update states

            states_value = [h, c]



    return decoded_sentence



label_list = []
pre_list = []



for seq_index in range(len(test_encoder_input_data)):
    input_seq = test_encoder_input_data[seq_index: seq_index + 1]
    wrf_input_seq = test_encoder_wrf_input_data[seq_index: seq_index + 1]

    decoded_sentence = decode_sequence(input_seq, wrf_input_seq)
    label_data = [i[0] for i in  test_decoder_target_data[seq_index]]
    
    if seq_index < 10:
        print('Label sentence:', label_data)
        # print('Label sentence:', decoder_target_data[seq_index])
        print('Decoded sentence:', decoded_sentence)

    label_list.append(label_data)
    pre_list.append(decoded_sentence)

print("########################")
# m.compute_mse_scalar(label_list, pre_list, y_scaler)
# m.compute_mse_winddir(label_list, pre_list, y_scaler)
m.draw_lstm_without_e(label_list, pre_list, y_scaler, lstm_path = global_path)




